#include <linux/module.h>
#include <linux/init.h>

#include <linux/fs.h>
#include <linux/ioctl.h>
#include <linux/kernel.h>
#include <asm/system.h>
#include <asm/segment.h>
#include <asm/io.h>
#include <linux/ioport.h>
#include <linux/pci.h>

#include "dpciindust.h"

#include <asm/uaccess.h>

/* I defined some macro for device_open function */
#define TYPE(dev) (MINOR(dev) >> 4) 
#define NUM(dev) (MINOR(dev) & 0x0f)

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(a,b,c) ((a)*65536+(b)*256+(c))
#endif

#ifndef DECISION_VENDOR_ID
#define DECISION_VENDER_ID 0x6666
#endif

#ifndef DECISION_DEVICE_ID
#define DECISION_DEVICE_ID 0x1011
#endif


static int dpciindust_cards;
static char dpciindust_rev[2];
int dpciindust_Base_Addr[MAXCARD];
//EXPORT_NO_SYMBOLS;
/*---------------------------------------------------------------------*/
int dpciindust_open(struct inode *inoder,struct file *flip)
{
/* int type= TYPE(inoder->i_rdev);*/

 int num=NUM(inoder->i_rdev);
 if(num>=dpciindust_cards) return -ENODEV;

// MOD_INC_USE_COUNT;
 return 0;
}
/*---------------------------------------------------------------------*/
static int dpciindust_release(struct inode *inoder,struct file *flip)
{
//  MOD_DEC_USE_COUNT;
  return 0;
}
/*---------------------------------------------------------------------*/
static ssize_t dpciindust_read(struct file *flip,char *buf,size_t count,loff_t *ptr)
{
 int countt=0;

  int num=MINOR(flip->f_dentry->d_inode->i_rdev);
  if(ptr!=&flip->f_pos)  /* can't seek on this device */
   return -ESPIPE;
 
 if(num>=dpciindust_cards)
   return -EINVAL;
 buf[0]=readb(dpciindust_Base_Addr[num]);
 countt=1;
           
 return countt;
}
/*---------------------------------------------------------------------*/
static ssize_t dpciindust_write(struct file *flip,const char *buf,size_t count,loff_t *ptr)
{
 if(ptr!=&flip->f_pos)  /* can't seek on this device */
  return -ESPIPE;
 
 return 0;       /* do nothing */
}
/*---------------------------------------------------------------------*/
int dpciindust_Myread(int num,char *buf,int count)
{
 int countt;
 outw(0xffff,dpciindust_Base_Addr[num]+((int)buf[0]<<1));
 
 countt=inw(dpciindust_Base_Addr[num]+((int)buf[0]<<1));
 buf[1]=(char)(countt&0xff);
 buf[2]=(char)(countt>>8);
 countt=2;
 return countt;
}
/*---------------------------------------------------------------------*/
void dpciindust_Mywrite(int num,char *buf,int count)
{
 int t;
 t=((int)buf[2])&0xff;
 t=t<<8;
 t=t+(((int)buf[1])&0xff);
 outw(t,dpciindust_Base_Addr[num]+(((int)buf[0])<<1));
}
/*---------------------------------------------------------------------*/
int dpciindust_ioctl(
    struct inode *inode,
    struct file *file,
    unsigned int ioctl_num,/* The number of the ioctl */
    unsigned long ioctl_param) /* The parameter to it */
{
  char param[4];
  char *temp=(char *)ioctl_param;
  char ch1=0;
  int data;
  int num=NUM(inode->i_rdev);
  switch (ioctl_num) 
   {
    case DPCIINDUST_SELECT_PORT:
      ch1=(char)((int)temp);
      dpciindust_rev[0]=ch1;
      break;

    case DPCIINDUST_WRITE_PORT:
      data=(int)temp;
      param[0]=dpciindust_rev[0];
      param[1]=(char)(data&0xff);
      param[2]=(char)(data>>8);
      dpciindust_Mywrite(num,param,3);
      break;

    case DPCIINDUST_READ_PORT:
      param[0]=dpciindust_rev[0];
      dpciindust_Myread(num,param,3);
      put_user(param[1],temp);
      temp++;
      put_user(param[2],temp);
      break;

     default:
             break;
  }

  return 0;
}
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
static struct file_operations dpciindust_fops={
.read		= dpciindust_read,
.write		= dpciindust_write,
.ioctl  	= dpciindust_ioctl ,
.open		= dpciindust_open,
.release	= dpciindust_release,
};

static int __init dpciindust_init(void)
{
 int index=0;
 struct pci_dev *dev = NULL;
 static int dev_count=0;
 int i; 

 dev_count++;
 if(dev_count>1)
  {
   printk(KERN_ERR"This drive only can open one time.\n");
   return -ENODEV;
  }

#if 0
#ifdef CONFIG_PCI
 if (!pci_present())   /* No PCI bus in this machine! */
 return -ENODEV;
#endif
#endif

 dpciindust_cards=0;
 while (index < MAXCARD && 
        (dev = pci_find_device(DECISION_VENDER_ID, DECISION_DEVICE_ID,dev))) 
 {
   dpciindust_Base_Addr[index]=pci_resource_start(dev,3) & PCI_BASE_ADDRESS_IO_MASK;   
   printk("Deciision PCI Industrial card:I got a base %x\n",dpciindust_Base_Addr[index]); 
   if(check_region(dpciindust_Base_Addr[index],REGIONLENGTH)>=0)
    request_region(dpciindust_Base_Addr[index],REGIONLENGTH,DEVICE_NAME);
   index++; 
  }

  if((dev==NULL)&&(index<1))
   {
    printk("\nI can't find any Decision PCI Industrial card here!!\n");
    return -ENODEV;
   }
 printk("\nDecision PCI Industrial card: Total %d cards that I catched!\n",index);
 dpciindust_cards=index;
 index=register_chrdev(dpciindust_major,DEVICE_NAME,&dpciindust_fops);
 if(index<0)
  {
  for(i=0;i<dpciindust_cards;i++)
   release_region(dpciindust_Base_Addr[i],REGIONLENGTH);
  return index;
  }
 dpciindust_rev[0]=0;
 dpciindust_rev[1]=0;
 printk("Decision PCI Industrial card:Register Successful\n");  
 return 0;

}
/*---------------------------------------------------------------------*/
static void __exit dpciindust_exit(void)
{
  int i;
  for(i=0;i<dpciindust_cards;i++)
   release_region(dpciindust_Base_Addr[i],REGIONLENGTH); 
  unregister_chrdev(dpciindust_major,DEVICE_NAME);
  printk("Decision PCI Industrial card:Unregister Successful!!\n");  
}
/*---------------------------------------------------------------------*/
module_init(dpciindust_init);
module_exit(dpciindust_exit);

/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
MODULE_AUTHOR("Delta");
MODULE_DESCRIPTION("Decision PCI Industrial Card Driver");
MODULE_LICENSE("GPL");
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
